#![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]

pub mod components;
pub mod constants;
mod graphics;
pub mod input;
pub mod levels;
pub mod resources;
mod state;
pub mod systems;
pub mod tile_data;

use std::time::Instant;

use bracket_lib::prelude::*;
use hecs::World;
use resources::{difficulty::SECRET, *};
use state::State;

fn main() -> BError {
    let context = BTermBuilder::vga(80, 25)
        .with_fps_cap(60.0)
        .with_title("Kroz")
        .build()?;

    let mut sound_output = SoundOutput::new();
    let sound_effects = SoundEffects::new(&sound_output);
    sound_output.play_sound(sound_effects.startup.clone());

    let starting_level = 0;
    let selected_difficulty = SECRET;

    let mut world = World::new();

    let mut map = Map::from_level(levels::get_level(starting_level));
    map.spawn_entities(&mut world);

    let resources = Resources {
        level_number: starting_level,
        show_debug_info: false,
        player_input: None,
        stats: Stats {
            score: 0,
            gems: selected_difficulty.starting_gems,
            whips: selected_difficulty.starting_whips,
            whip_power: selected_difficulty.starting_whip_power,
            teleports: selected_difficulty.starting_teleports,
            keys: selected_difficulty.starting_keys,
        },
        clock: Clock {
            last_ticked: Instant::now(),
            has_ticked: false,
            ticks: 0,
        },
        stop_clock: false,
        map,
        sound_effects,
        sound_output,
        selected_difficulty: Some(selected_difficulty),
        flashing_message: None,
    };

    // let descent_sounds: Vec<Sound> = (20..100)
    //     .rev()
    //     .flat_map(|x| {
    //         (1..10).rev().flat_map(move |y| {
    //             vec![Sound {
    //                 sound_type: SoundType::Tone(x * y * y),
    //                 duration: Duration::from_millis((y as f64 / 1.5) as u64),
    //             }]
    //         })
    //     })
    //     .collect();

    // let descent_effect = gs.sound_system.render_sound_effect(SoundEffect {
    //     sounds: descent_sounds,
    // });

    // let _ = gs.sound_system.play_sound(descent_effect);

    main_loop(context, State::new(world, resources))
}
